/* 
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *         (C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include <stdlib.h>
#include <moddisksetup.h>
#include <moddisksel.h>
#include <variables.h>
#include <translation.h>
#include <dialogs.h>
#include <dbsdglobal.h>
#include <dbsdiconloader.h>

void frmInstallationWizard::reboot() {
	/* Don't use reboot(2) here, because reboot(1) does
	 * some valuable preparations like syncing buffers
	 * and trying to shutdown processes tidily.
	 */
	system("/sbin/reboot");
}


void frmInstallationWizard::init() {
	DBSDIconLoader *il = DBSDGlobal::getIconLoader();
	pixMain->setPixmap(il->loadIcon("desktopbsd"));
	pixLangSelect->setPixmap(il->loadIcon("locale"));
	pixBootSector->setPixmap(il->loadIcon("bootsector"));
	pixCopy->setPixmap(il->loadIcon("copy"));
	pixPartition->setPixmap(il->loadIcon("partition"));
	pixSuccess->setPixmap(il->loadIcon("success"));
	pixFailure->setPixmap(il->loadIcon("error"));
	btnReboot->setIconSet(il->loadIcon("reboot"));
	btnReboot2->setIconSet(il->loadIcon("reboot"));

   /* A short help text is displayed on every page */
	helpButton()->hide();
	
	QFile f("/usr/local/share/desktopbsd/install/languages");
	if(f.open(IO_ReadOnly)) {
		QTextStream s(&f);
		s.setEncoding(QTextStream::UnicodeUTF8);
		
		while(!s.atEnd()) {
			QStringList lang = QStringList::split(':', s.readLine());
			
			langCodes.append(lang[0]); /* ISO 639-1 code, e.g. "de" */
			lstLanguage->insertItem(lang[1]); /* Native name, e.g. "Deutsch" */
		}
		
		f.close();
	}

	setBackEnabled(pgWelcome, false);
}


void frmInstallationWizard::pageChanged() {
	QWidget *cpage = currentPage();

	if(cpage == pgWelcome) {
		if(lang.isEmpty())
			setLanguage();

		erase();
	} else if(cpage == pgDiskSel) {
		/* Create disk selector */
		delete modSelDisk;
		modSelDisk = new modDiskSel(pgDiskSel);
		pgDiskSelLayout->addWidget(modSelDisk, 1, 0);
		modSelDisk->initializeModule();
		modSelDisk->show();
	} else if(cpage == pgBootLoader) {
      /* When going back from the disk partitioner */
      nextButton()->show();

		if(modSelDisk != NULL)
			modSelDisk->apply();

		delete modSelDisk;
		modSelDisk = NULL;
	} else if (cpage == pgDiskPart) {
      nextButton()->hide();

		Variables::setValue("disk.bootLoader", QString::number(grpLoader->selectedId()));

		delete modPartDisk;
		modPartDisk = new modDiskSetup(pgDiskPart);
		pgDiskPartLayout->addMultiCellWidget(modPartDisk, 1, 1, 0, 1);
		modPartDisk->initializeModule();
		modPartDisk->show();
		connect(modPartDisk, SIGNAL(done()), SLOT(next()));
	} else if(cpage == pgCopy) {
      backButton()->hide();
		nextButton()->hide();

		delete modPartDisk;
		modPartDisk = NULL;
		
		uint count;
		bool countOk = false;
		QFile f("/usr/local/share/desktopbsd/install/filecount");
		if(f.open(IO_ReadOnly)) {
			QTextStream s(&f);
			count = s.readLine().toUInt(&countOk);
			f.close();
		}
			
      prgrCopy->setTotalSteps(countOk ? count : 0);
      
		startCopy();
	} else if(cpage == pgFinish) {
		backButton()->hide();
		nextButton()->hide();
		cancelButton()->hide();
		if(!cproc->normalExit() || cproc->exitStatus()) {
			next();
			lblExactError->setText(lastLine);
			delete cproc;
		}

		/* -- Do nothing -- */
	} else if(cpage == pgError) {
		backButton()->hide();
		nextButton()->hide();
		cancelButton()->hide();
	}
}


void frmInstallationWizard::startCopy() {
	cproc = new QProcess(QString("/usr/local/share/desktopbsd/install/install.sh"));
	
	/* [1] Disk to install to */
	cproc->addArgument(Variables::getValue("disk.activeDisk"));

	/* [2] Disk and slice to install to (set by modPartDisk */
	cproc->addArgument(Variables::getValue("disk.activeSlice"));

	/* [3] Installation language. Creates a file for the "Initial
	 * Configuration Wizard" which can then read the installation
	 * language and doesn't have to ask the user about this.
	 */
	cproc->addArgument(lang);

   cproc->setCommunication(QProcess::Stdout | QProcess::Stderr | QProcess::DupStderr);

 	QObject::connect(cproc, SIGNAL(readyReadStdout()), SLOT(copyActivity()));
	connect(cproc, SIGNAL(processExited()), SLOT(next()));
	cproc->start();
}


/* Previously used to display the script's output */
void frmInstallationWizard::copyActivity() {
	static int progress = 0;

	QString line, previous;
	while(!(line = cproc->readLineStdout()).isNull()) {
		prgrCopy->setProgress(++progress);
		previous = line;
	}

	lastLine = previous;
}



void frmInstallationWizard::setLanguage() {
	QListBoxItem *lbi = lstLanguage->selectedItem();

	if(lbi != NULL) {
		int index = lstLanguage->index(lbi);
		
		if(index > 0) {
			/* Not English */
			lang = langCodes[index - 1];
			Translation::installTranslator(lang);
			Translation::translateWizard(this);
		} else
			lang = "en";
	}
}


void frmInstallationWizard::reject() {
	if(Dialogs::question(tr("Are you sure you want to cancel the installation?\n"
									"\n"
									"Click 'Yes' to confirm and reboot the computer or\n"
									"'No' to continue installing DesktopBSD.")))
		/* Don't clean up, everything is lost :) */
		reboot();
}

