/* 
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *         (C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include <dbsdglobal.h>
#include <netinterface.h>
#include <wirelessinterface.h>
#include <ipv4netmask.h>

/* Qt */
#include <qtimer.h>
#include <qprocess.h>
#include <qtooltip.h>

/* KDE */
#include <kiconloader.h>
#include <kapp.h>
#include <kpopupmenu.h>
#include <kpassivepopup.h>
#include <kmessagebox.h>

#include "nettray.h"

/* Update interval in ms */
#define  UPDATE_MSEC 1500

NetworkTray::NetworkTray() {
	setPixmap(getIcon());

	contextMenu()->insertItem(DBSDGlobal::getIconLoader()->loadIcon("configure", DBSDIconLoader::Size16x16),
	                          tr("Configure..."), this, SLOT(openConfigDlg()));

	updateStatus();
}

void NetworkTray::mouseDoubleClickEvent(QMouseEvent *e) {
	if(e != NULL) {
		openConfigDlg();
	}
}

QPixmap  NetworkTray::getIcon() {
	return  DBSDGlobal::getIconLoader()->loadIcon("dbsd-nettray", DBSDIconLoader::Size22x22);
}

void  NetworkTray::openConfigDlg() {
	QProcess  process(QStringList::split(' ', "/usr/local/bin/kdesu /usr/local/bin/kcmshell dbsdnetworking"));
	if(!process.start())
		KMessageBox::error(this,  tr("Cannot start DesktopBSD Networking configuration."));
}

QString  NetworkTray::statusToStr(NetInterface::Status  status) {
	switch(status) {
		case  NetInterface::Active:
		return  tr("Active");

		case  NetInterface::NoCarrier:
		return  tr("No Carrier");

		case  NetInterface::Associated:
		return  tr("Associated", "Wireless device associated to an access point");

		default:
		return  tr("(Error)");
	}
}

void  NetworkTray::updateIntf(NetInterface  *iface) {
	QString  ifName = iface->getName();
	QPair<IPv4Addr, NetInterface::Status>  oldStatus,
														newStatus;
														
	newStatus = qMakePair(iface->getIP(),  iface->getStatus());

	if(ifStatus.contains(ifName)) {
		oldStatus = ifStatus[ifName];

		QString  message;
		if((!oldStatus.first.isValid() && newStatus.first.isValid()) || oldStatus.first != newStatus.first)
			message += tr("IP Address changed to <i>%1</i>.").arg(newStatus.first.toString());
			
		if(oldStatus.second != newStatus.second) {
			if(!message.isEmpty())
				message += "<br>";
		
			message += tr("Link status changed to <i>%1</i>.").arg(statusToStr(newStatus.second));
		}
			
		if(!message.isEmpty())
			KPassivePopup::message(iface->getDesc(), message,  getIcon(),  this);
	}
	
	ifStatus[ifName] = newStatus;
}

void  NetworkTray::updateStatus() {
	QPtrList<NetInterface>  ifList =  NetInterface::getIfs(true, false);

	QString  tooltipStr;

	NetInterface  *iface = ifList.first();
	while(iface != NULL) {
		updateIntf(iface);

		QString desc = iface->getDesc();
		if(desc.length() >= 28) {
			desc.truncate(28);
			desc += "...";
		}

		NetInterface::Status  status = iface->getStatus();
		tooltipStr += QString(tr("<b>%1</b><br>Status: <i>%2</i>%3"))
										.arg(desc)
										.arg(statusToStr(status))
										.arg((status == NetInterface::Active || status == NetInterface::Associated) ?
											  tr("<br>%1IP Address: <i>%4</i><br>Netmask: <i>%5</i>")
												.arg(iface->isWireless() ? tr("Network (SSID): <i>%1</i><br>").arg(((WirelessInterface *) iface)->getSSID()) : "")
												.arg(iface->getIP().toString())
												.arg(iface->getNetmask().toString()) : "");
		

		if((iface = ifList.next()) != NULL)
			tooltipStr += "<br><br>";
	}

	QToolTip::add(this, tooltipStr);

	ifList.setAutoDelete(true);
	QTimer::singleShot(UPDATE_MSEC,  this,  SLOT(updateStatus()));
}
