/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *			(C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include <dbsdglobal.h>
#include <dbsdiconloader.h>

/* Qt */
#include <qpixmap.h>
#include <qbitmap.h>

#include <actionlist.h>



ActionList::ActionList(QWidget  *parent,  const char *name,  WFlags  flags)
: QListView(parent, name, flags)
{
	QPalette  pal = palette();
	pal.setDisabled(pal.active());
	setPalette(pal);

	currentAction = NULL;

	setEnabled(false);
	setSelectionMode(Single);
	setSorting(-1);

	addColumn("", -1);
	setResizeMode(AllColumns);

	numSuccessfulActions = 0;
	numFailedActions = 0;
}



/** Returns the number of failed actions so far.
  *
  */
int  ActionList::getNumFailedActions() {
	return  numFailedActions;
}



/** Returns the number of failed actions so far.
  *
  */
int  ActionList::getNumSuccessfulActions() {
	return  numSuccessfulActions;
}



/** Inserts a new action in the list.
 *
 * @param position Predecessor of this action.
 * @param parent	Parent action of this action.
 *
 * If no precedessor is specified or its value is less
 * than 0, the action will be appended to the list.
 *
 * If parent is not specified, the action will become
 * a top-level action.
 *
 * @return The index of the created item.
 */
int  ActionList::addAction(QString  action,  int  parent,  int  precedessor) {
	QListViewItem	*item,
						*pPrev,
						*pParent = findItem(parent);

	if(pParent != NULL) {
			if((pPrev = findItem(precedessor)) == NULL)
				pPrev = findLastItem(pParent);

			item = new QListViewItem(pParent, pPrev, action);
			pParent->setPixmap(0,  DBSDGlobal::getIconLoader()->loadIcon("folder", DBSDIconLoader::Size22x22));
	} else {
			pPrev = findItem(childCount() - 1);
			item = new QListViewItem(this, pPrev, action);
	}

	item->setOpen(true);
	item->setPixmap(0,  DBSDGlobal::getIconLoader()->getBlankPixmap(22, 22));

	processAction(firstChild());
	return  getIndex(item);
}



/**
 * Marks the current action as successful and
 * the following action as currently processed.
 */
void  ActionList::actionSuccessful() {
	actionSuccessfulStop();
	processNextAction();
}



/**
 * Marks the current action as failed and
 * the following action as currently processed.
 */
void  ActionList::actionFailed() {
	actionFailedStop();
	processNextAction();
}


/** Marks the current action as successful,
  * but does not mark the next action as
  * currently processed.
  */
void  ActionList::actionSuccessfulStop() {
	if(currentAction != NULL) {
		currentAction->setPixmap(0,  DBSDGlobal::getIconLoader()->loadIcon("button_ok",
																								 DBSDIconLoader::Size22x22));

		++numSuccessfulActions;
	}
}



/** Marks the current action as failed.
  * but does not mark the next action as
  * currently processed.
  */
void  ActionList::actionFailedStop() {
	if(currentAction != NULL) {
		currentAction->setPixmap(0,  DBSDGlobal::getIconLoader()->loadIcon("button_cancel",
																								 DBSDIconLoader::Size22x22));

		++numFailedActions;
	}
}



/** Returns the index of the currently
* processed action.
*/
int  ActionList::currentIndex() {
	return  getIndex(currentAction);
}




/** Marks the next action as currently
  * processed action or goes higher in
  * the hierarchy if this was the last
  * subitem.
  */
void  ActionList::processNextAction() {
	if(currentAction == NULL)
		return;

	QListViewItem  *nextItem = currentAction->nextSibling();

	QListViewItem  *parent = currentAction;
	while(nextItem == NULL  &&  parent != NULL) {
		parent = parent->parent();
		nextItem = (parent == NULL) ? NULL : parent->nextSibling();
	}

	processAction(nextItem);
}



void  ActionList::processAction(QListViewItem  *action) {
	if(action != NULL) {
		while(action->firstChild() != NULL)
			action = action->firstChild();

		setSelected(action,  true);
		currentAction = action;
	}
}



QListViewItem  *ActionList::findItem(int  index) {
	if(index < 0)
		return  NULL;

	QListViewItemIterator  it(this);
	for(int  i = 0;  i < index && *++it;  ++i);
	return  *it;
}


/**
  * Returns a pointer to the last child of the
  * given item or NULL when the item has no
  * children or is NULL.
  */
QListViewItem  *ActionList::findLastItem(QListViewItem  *item) {
	if(item == NULL)
		return  NULL;

	item = item->firstChild();
	if(item != NULL) {
		QListViewItem  *next;
		while((next = item->nextSibling()) != NULL)
			item = next;
	}

	return  item;
}



int  ActionList::getIndex(QListViewItem  *srchItem) {
	if(srchItem == NULL)
		return  -1;

	int  pos;
	QListViewItemIterator  it(this);
	for(pos = 0;  *it != NULL  &&  *it != srchItem;  ++pos, ++it);
	return  pos;
}
