/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *			(C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include <dbsdglobal.h>
#include <dbsdiconloader.h>
#include <dialogs.h>

void  frmTerminal::init() {
	successMessage =  tr("The operation finished successfully.\n"
								"\n"
								"You can scroll with the mouse wheel.\n"
								"Click 'Close' when you're done.");

	failureMessage = tr("The operation failed.\n"
							  "\n"
							  "You can scroll with the mouse wheel to find out what the\n"
							  "error was. Click 'Close' when you're done.");
							  
	verbose = true;
	
	DBSDIconLoader *iloader = DBSDGlobal::getIconLoader();
	btnCancel->setIconSet(iloader->loadIcon("button_cancel", DBSDIconLoader::Button));
	connect(terminal, SIGNAL(exited()), SLOT(processDone()));
}


/**
  * Use this method to prevent the user from stopping
  * the running process.
  *
  *
  */
void frmTerminal::setCancelEnabled(bool enabled) {
	btnCancel->setEnabled(enabled);
}


/**
  * When the user clicks the 'close' button and the program is
  * still running, he will be asked if he wants to kill the
  * program. When he chooses to kill it, the result code is
  * QDialog::Rejected.
  *
  * When no process is running, the result code is QDialog::Accepted.
  *
  * To get the exit code of the program, use getTerminal() and
  * Terminal::exitStatus().
  */
void frmTerminal::closeEvent(QCloseEvent *e) {
	if(terminal->isRunning()) {
		if(Dialogs::question(tr("Are you sure?"))) {
			terminal->close();
			setResult(Rejected);
			e->accept();
		} else
			e->ignore();
	} else {
		setResult(Accepted);
		e->accept();
	}
}


/**
  * Returns the terminal used in the dialog.
  *
  * DO NOT call start() manually, this will be
  * done automatically when the dialog is about
  * to be displayed.
  *
  * If the terminal fails to start, the dialog
  * will close immediately and exec will return
  * QDialog::Rejected.
  */
Terminal * frmTerminal::getTerminal() {
	return terminal;
}


void frmTerminal::polish() {
	QDialog::polish();

	if(terminal->start() < 0)  {
		/* Disable verbose mode; this is a terminal error and
		 * verbose messages are related to the processes started
		 * in the terminal
		 */
		bool wasVerbose = isVerbose();
		setVerbose(false);
		processDone();
		setVerbose(wasVerbose);
	}
}


void frmTerminal::processDone() {
	DBSDIconLoader *iloader = DBSDGlobal::getIconLoader();
	btnCancel->setText(tr("Close"));
	btnCancel->setIconSet(iloader->loadIcon("fileclose", DBSDIconLoader::Button));

	if(verbose) {
		if(terminal->exitStatus() == 0)
			Dialogs::infoDlg(successMessage);
		else
			Dialogs::warnDlg(failureMessage);
	}
}



/** Activates or deactivates verbose output (default: activated).
  *
  * When verbose output is on, a message is displayed that
  * tells the user whether the operations succeeded or failed.
  * 
  * You can set custom success/failure messages with
  * setFailureMessage() and setSuccessMessage().
  */
void frmTerminal::setVerbose(bool v) {
	verbose = v;
}


bool frmTerminal::isVerbose() {
	return verbose;
}


void frmTerminal::setFailureMessage(const QString & msg) {
	failureMessage = msg;
}


QString frmTerminal::getFailureMessage() {
	return failureMessage;
}


void frmTerminal::setSuccessMessage(const QString & msg) {
	successMessage = msg;
}


QString frmTerminal::getSuccessMessage() {
	return successMessage;
}
