/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *			(C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include "terminal.h"

#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include <qtimer.h>

#include <environment.h>
#include <unixprocess.h>
#include <frmtextbrowser.h>

#include <X11/Xlib.h>

#define XTERM_CMD				"/usr/X11R6/bin/xterm"
#define PGRPMGR_CMD			"/usr/local/bin/pgrpmgr"

/* Must be fixed-width */
#define FONT_RAW	"-*-lucidatypewriter-medium-*-*-*-*-120-*-*-*-*-*-*"


Terminal::Terminal(QWidget *p, const char *n):
	QLabel(p, n),
	cols(80), rows(30),
	termProcess(NULL),
	exitFIFO(-255),
	exitFIFOPath(NULL),
	ioTimer(NULL),
	exitCode(INT_MIN)
{
}

Terminal::~Terminal() {
	close();
}

/** Terminates the whole terminal process group.
  *
  * @returns TRUE when the signal was sent (independent of the
  *			 program's reaction), FALSE when no terminal is open.
  *
  * @see Terminal::start()
  */
bool Terminal::tryTerminate() {
	if(termProcess != NULL && termProcess->isRunning()) {
		termProcess->tryTerminate();
		return true;
	}

	return false;
}

/** After termination of the program running in the terminal,
  * the terminal itself is still open. When a close event arrives,
  * terminate the terminal process.
  */
void Terminal::closeEvent(QCloseEvent *e) {
	tryTerminate();
	e->accept();
}

void Terminal::mouseDoubleClickEvent(QMouseEvent *e) {
	if(!errorDetails.isEmpty()) {
		frmTextBrowser *dlg = new frmTextBrowser(this, NULL, true, QWidget::WDestructiveClose);
		dlg->setDisplayText(tr("PLEASE INCLUDE THIS INFORMATION IN ANY BUG REPORTS.\n\n%1").arg(errorDetails));
		dlg->setCaption(tr("Error details"));
		dlg->show();

		e->accept();
	} else
		QLabel::mouseDoubleClickEvent(e);
}

/**
  * Returns TRUE if the process in the terminal is still
  * running, otherwise FALSE.
  *
  */
bool Terminal::isRunning() {
	return (exitFIFO != -1);
}

/** Returns the exit code of the last process that was
  * started in this terminal. If no process was started
  * before, this returns -255. If the process hasn't
  * terminated yet, this returns the return value of
  * start().
  *
  * @see QProcess::exitStatus()
  */
int Terminal::exitStatus() {
	return exitCode;
}

/** Sets the geometry. First parameter is the column
  * count, second parameters row count.
  *
  * This has only effect if the values are set BEFORE
  * the process is started.
  */
bool Terminal::setContentGeometry(int c, int r) {
	if(c > 0 && r > 0) {
		cols = c;
		rows = r;
		return true;
	}

	return false;
}

/**
  * Displays an error text.
  *
  * @param details Displayed when the user double-clicks on the widget.
  */
void Terminal::setErrorText(const QString & msg, const QString & details) {
	setText(tr("An error has occured: %1\n"
				  "Please restart the package manager and try again.")
				.arg(msg));

	errorDetails = details;
}

/** Launches the terminal that will be displayed in the widget.
  *
  * @return A value < 0 if an error occured, otherwise 0.
  *
  * Return values:
  * 
  * -1: No process given
  * -2: FIFO creation in /tmp failed
  * -3: Opening FIFO for reading failed
  * -4: Starting the process failed
  * -5: Process started, but exited before opening a terminal
  * 0: Success
  */
int Terminal::start() {
	if(termProcess == NULL) {
		setErrorText("No terminal process specified.");
		return (exitCode = -1);
	}

	connect(termProcess, SIGNAL(processExited()), SLOT(termProcessExited()));

	/* Create a temporary named pipe (FIFO).
	 *
	 * When the process exits, it's exited status followed
	 * by '#' (as termination symbol) will be written into
	 * the file. By this, we know about the processes exit
	 * status and when it exited.
	 */
	char s[] = { "/tmp/dbsdterminal.XXXXXX" };
	if(mkfifo(mktemp(s), 0600) < 0) {
		setErrorText("FIFO creation in /tmp failed.", QString("mkfifo() errno: %1").arg(errno));
		return (exitCode = -2);
	}
	
	QStringList args;
	args << XTERM_CMD << "-hold" << "-geometry" << QString("%1x%2").arg(cols).arg(rows)
		  << "-j" << "-fn" << FONT_RAW << "-into" << QString::number(winId())
		  << "-b" << "0" << "-e"
		  << PGRPMGR_CMD << "-o" << s;
	termProcess->setArguments(args + termProcess->arguments());
	qDebug("Terminal::start <'%s'>", termProcess->arguments().join("' '").latin1());
	
	/*
	 * Add our own environment variables, otherwise locale
	 * settings and the xterm itself wouldn't work.
	 * 
	 * If one of these variables was already set 
	 * 
	 */
	
	if(termProcess->isEnvironmentSet()) {
		/* Remove environment variables we would overwrite */
		QStringList env = termProcess->getEnvironment();
		
		QStringList curEnv = Environment::getCurrentEnv();
		QStringList::Iterator i = curEnv.begin();
		while(i != curEnv.end()) {
			QStringList::Iterator ie = env.begin();
			
			if((*i).startsWith((*ie).section('=', 0, 0) + '='))
				i = curEnv.remove(i);
			else
				++i;
		}
		
		termProcess->setEnvironment(env + curEnv);
	}

	/* 
	 * FIXME
	 * 
	 * This is a quite bad solution, but Qt 3 doesn't
	 * provide good methods to check the avalability
	 * of data in a QIODevice and QThread has no
	 * signals to indicate termination.
	 *
	 * Use QTimer to check if new data is available in
	 * the pipe every 10 milliseconds.
	 */
	exitFIFO = open(s, O_RDONLY | O_NONBLOCK);
	if(exitFIFO < 0) {
		setErrorText("Opening the FIFO in /tmp for reading failed.", QString("open() errno: %1").arg(errno));
		return (exitCode = -3);
	}

	exitFIFOPath = new QCString(s);
	ioTimer = new QTimer(this);
	connect(ioTimer, SIGNAL(timeout()), SLOT(checkIOAvailable()));

	int success = termProcess->start() ? 0 : -4;
	
	if(success == 0) {		
		/* Wait for the xterm window to be opened and use it's
		 * size as our own widget size.
		 */
		Display *dsp = XOpenDisplay(NULL);
		Window wnd = winId();

		bool childFound = false;
		while(!childFound && termProcess->isRunning()) {
			Window root, parent, *children;
			uint numwin;
			XQueryTree(dsp, wnd, &root, &parent, &children, &numwin);
			childFound = (children != NULL);
			
			if(childFound) {
				XWindowAttributes xwa;
				XGetWindowAttributes(dsp, *children, &xwa);

				QSize size(xwa.width, xwa.height);
				setMinimumSize(size);
				setMaximumSize(size);
			
				XFree(children);
			}
		}

		XCloseDisplay(dsp);

		if(!childFound)
			success = -5;
		else
			ioTimer->start(10, true);
	}


	if(success < 0) {
		cleanUp();
		setErrorText((success == -4)
							? "Starting the process failed"
							: "Process started, but exited before opening a terminal");

		errorDetails = QString("[Command]\n"
						 			  "%1\n\n"
						 			  "[Environment]\n"
						 			  "%2")
								.arg("'" + termProcess->arguments().join("' '") + "'")
								.arg(termProcess->getEnvironment().join("\n"));
	}
	
	return (exitCode = success);
}

/**
  * Cleans up after the terminal terminated or starting it failed.
  */
void Terminal::cleanUp() {
	delete ioTimer;
	ioTimer = NULL;

	::close(exitFIFO);
	exitFIFO = -1;

	unlink(*exitFIFOPath);
	delete exitFIFOPath;
	exitFIFOPath = NULL;
}

/**
  * Checks for new content in the exit FIFO every 10 msecs.
  *
  */
void Terminal::checkIOAvailable() {
	static QCString exitFIFOContent;

	if(exitFIFO != -1) {
		char buffer[16];
		
		int cnt = read(exitFIFO, buffer, sizeof buffer - 1);
		if(cnt > 0) {
			buffer[cnt] = '\0';
			exitFIFOContent.append(buffer);

			int pos = exitFIFOContent.find('#');
			if(pos >= 0) {
				exitCode = exitFIFOContent.left(pos).toInt();
				exitFIFOContent = "";
				cleanUp();
				emit exited();
			}

		}

		if(ioTimer != NULL)
			ioTimer->start(10, true);
	}
}

void Terminal::termProcessExited() {
	delete termProcess;
	termProcess = NULL;
}

/**
  * Sets a process that should be run in a terminal. If the terminal
  * is still being displayed, close it using close() or this function
  * will return false.
  *
  */
bool Terminal::setProcess(const UnixProcess &p) {
	if(termProcess != NULL)
		return false;

	termProcess = new UnixProcess;
	termProcess->set(p);
	return true;
}
