/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Authors: Peter Hofer <hofer.p@gmail.com>
 *          Reinhard Pointner
 *          (C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#ifndef __PACKAGE_H
#define __PACKAGE_H

#include <size.h>

#include <qstring.h>
#include <qstringlist.h>

#include <sys/param.h> /* __FreeBSD_version */

template <class T> class QPtrList;

/* Keep the slashes at the end of
 * the directory paths!
 */
#define	PORTSDIR	"/usr/ports/"
#define	PKGDB		"/var/db/pkg/"

#if defined(__FreeBSD_version) && __FreeBSD_version >= 600000
#define INDEX_PATH	"/usr/ports/INDEX-6"
#elif defined(__FreeBSD_version) && __FreeBSD_version >= 500036
#define INDEX_PATH	"/usr/ports/INDEX-5"
#else
#error "FreeBSD >= 5 required"
#endif

/** @brief Port from the FreeBSD ports collection.
  *
  * This class provides functionality to acquire information
  * about available or installed ports.
  *
  * Because the ports collection is very large, some properties
  * from the ports system are not used because of the large
  * data amount (~ 14000 objects when all packages are read in).
  */
class Package {
public:
	enum InstallStatus {
		Installed,
		InstalledOutdated,
		InstalledNewer,
		NotInstalled,
		Undefined
	};

	QString		getName()				const	{ return name;						};
	QString		getWebsite()			const	{ return website;					};
	QString		getLatestVersion()	const	{ return latestVersion;			};
	QString		getInstalledVersion()const	{ return installedVersion;		};
	QString		getShortDescr()		const	{ return shortDescr;				};
	QString		getMainCategory()		const	{ return categories.first();	};
	size_t		getRequiredByCount()	const	{ return requiredBy.count();	};
	QStringList	getCategories()	const	{ return categories;				};
	QStringList	getRequiredBy()	const	{ return requiredBy;				};
	InstallStatus	getInstallStatus()	const	{ return instStatus;				};

	QString				getID()					const;
	QString				getLongDescr();

	bool		isRequiredBy(const Package &);

	bool		isInstalledSizeSet();
	Size		getInstalledSize();

	void	mergeWithLatestPackage(const Package &);

	void  setInstallStatus(InstallStatus  instStatus)  { this->instStatus = instStatus;		};
	void	setInstalledVersion(const QString & version)	{ installedVersion = version;			};
	void	setLatestVersion(const QString & version)		{ latestVersion = version;				};

	static Package  *fromINDEXLine(const QString &);
	static Package  *fromPkgDb(const QString &);

	static QPtrList<Package>	getInstalledPackages();

private:
	Package();

	QString	name;
	QString	installedName;
	QString	installedVersion;
	QString	latestVersion;
	QString	shortDescr;
	QString	website;
	QStringList  categories;
	QStringList  requiredBy;
	InstallStatus  instStatus;

	Size	installedSize;
	bool	installedSizeSet;

	QString	longDescr;
	bool		longDescrSet;

	/* Not used in UI

	QString	mail;
	QString	portPath;
	*/
};


#endif /* __PACKAGE_H */
