/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *         (C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#ifndef __SYSCTL_H
#define __SYSCTL_H

/* strlen() */
#include <cstring>

/* Qt */
#include <qstring.h>

/* sysctl */
#include <sys/types.h>
#include <sys/sysctl.h>

/** @brief Read and write sysctl values. */
class SysCtl {
public:
    /** Sets an integer sysctl variable.
      * @return TRUE when sysctlbyname() succeeded,
      *         otherwise FALSE.
      */
    static bool  setIntCtl(const char *ctl, int value) {
        return  sysctlbyname(ctl, NULL, NULL, &value, sizeof(value)) == 0;
    };

    /** Sets a string sysctl variable.
      * @return TRUE when sysctlbyname() succeeded,
      *         otherwise FALSE.
      */
    static bool  setStrCtl(const char *ctl, const char *value) {
        return sysctlbyname(ctl, NULL, NULL, (void*) value, strlen(value)) == 0;
    };

    /** @return The value of a sysctl integer variable or
      *         -1 if sysctlbyname() failed.
      */
    static int  getIntCtl(const char *ctl) {
        int     value = -1;
        size_t  len = sizeof(value);
        sysctlbyname(ctl, &value, &len, NULL, 0);
        return  value;
    };

    /** @return The value of a sysctl string variable
      *         or QString::null if sysctlbyname() failed.
      */
    static QString  getStrCtl(const char *ctl) {
        return QString(getCStrCtl(ctl));
    };

	/** @returns The value of a sysctl string variable
	  *          as a CString. Only use this for internal
	  *          representation, otherwise use getStrCtl().
	  */
    static QCString  getCStrCtl(const char *ctl) {
        char    buffer[512];
        size_t  len = sizeof(buffer);
        if(sysctlbyname(ctl, buffer, &len, NULL, 0) == 0) {
            buffer[len] = '\0';
            return buffer;
        } else
            return (char *) NULL;
    };

	/** Returns TRUE when the specified sysctl exists
	  * in the system, otherwise false.
	  */
	static bool ctlExists(const char *ctl) {
		int  mib[8];
		size_t len = sizeof mib / sizeof(mib[0]);
		return (sysctlnametomib(ctl, mib, &len) == 0);
	}
};

#endif /* __SYSCTL_H */
