/*
 * This file is part of DesktopBSD, see the README file.
 *
 * Author: Peter Hofer <hofer.p@gmail.com>
 *			(C) 2004, 2005
 *
 * Copyright: BSD, see the COPYING file included in this distribution
 *
 */

#include "wirelessinterface.h"

#include <net/ethernet.h>
#include <net/if_dl.h>
#include <net/if_types.h>
#include <sys/ioctl.h>
#include <sys/socket.h>
#include <net/if_media.h>
#include <sysctl.h>
#include <ifaddrs.h>
#include <net/route.h>
#include <net/if.h>
#include <net80211/ieee80211.h>
#include <net80211/ieee80211_crypto.h>
#include <net80211/ieee80211_ioctl.h>

#include <qstring.h>
#include <qptrlist.h>

#include <wlan.h>
#include <accesspoint.h>
#include <executor.h>

WirelessInterface::WirelessInterface(const  QString &  devName)
	: NetInterface(devName)
{
}

bool  WirelessInterface::connectTo(const  WLAN &  wlan) {
	WEPMode	wepmode = wlan.getWEPMode();

	return  (setSSID(wlan.getSSID()) &&
				setWEPMode(wepmode)  &&
				(wepmode != WEPOn || (setWEPKeyIndex(1) && setWEPKey(wlan.getWEPKey()))) &&
				NetInterface::connectTo(wlan));
}

/** Sets the device's SSID.
  * (Maximum length is 32 characters)
  */
bool  WirelessInterface::setSSID(const  QString &  ssid) {
	uint8_t  data[33];
	if(strlcpy((char *) data,  ssid.latin1(),  sizeof  data) >= sizeof data)
		return  false;

	return  (set80211(IEEE80211_IOC_SSID,  0,  ssid.length(),  data) >= 0);
}

bool  WirelessInterface::setWEPMode(WEPMode  mode) {
	int  wmode;
	switch(mode) {
	case WEPOn:
		wmode = IEEE80211_WEP_ON;
		break;

	case WEPMixed:
		wmode = IEEE80211_WEP_MIXED;
		break;

	case WEPOff:
		wmode = IEEE80211_WEP_OFF;
		break;

	default:
		return  false;
	}

	return  (set80211(IEEE80211_IOC_WEP, wmode, 0, NULL) >= 0);
}

int  WirelessInterface::hexToNumber(char  chr)  const {
	if (chr >= '0' && chr <= '9')
		chr -= '0';
	else if (chr >= 'A' && chr <= 'F')
		chr -= ('A' - 10);
	else if (chr >= 'a' && chr <= 'f')
		chr -= ('a' - 10);
	else
		chr = 0;

	return  chr;
}

/** @param key Hexadecimal key without prepended '0x'
  *				or an empty string to unset the current key.
  *
  *				This does NOT change the WEP mode (On, Off, ...)
  */
bool  WirelessInterface::setWEPKey(const QString &  key) {
	uint8_t  data[IEEE80211_KEYBUF_SIZE];
	bzero(data,  sizeof data);

	/* Convert key from hex to string */
	const char  *asc = key.latin1();
	if(asc == NULL)
		return  false;
	uint	i;
	for(i = 0;  asc[i*2] != '\0'  &&  asc[i*2+1] != '\0';  ++i) {
		if(!isxdigit(asc[i*2])  ||  !isxdigit(asc[i*2+1]))
			return  false;

		data[i] = (hexToNumber(asc[i*2]) << 4) | hexToNumber(asc[i*2+1]);
	}

	return  (set80211(IEEE80211_IOC_WEPKEY,  /* Key Index */ 0,  i,  data) >= 0);
}


/**
  * Set the default key to use for transmission. If the given
  * index is 0, the used key index is set to 'undefined'.
  *
  * @param index The key index, an integer value >= 1.
  */
bool WirelessInterface::setWEPKeyIndex(int index) {
	if(index-- == 0)
#ifdef IEEE80211_KEYIX_NONE
		index = IEEE80211_KEYIX_NONE;
#else
		index = -1;
#endif

	return (set80211(IEEE80211_IOC_WEPTXKEY, index, 0, NULL) >= 0);
}


/** Returns the current SSID of the interface
  * (can be != desired SSID)
  */
QString  WirelessInterface::getSSID()  const {
	uint8_t  data[32];
	struct ieee80211req  *ireq = getIReq(IEEE80211_IOC_SSID,  (uint8_t **) &data);
	if(ireq == NULL)
		return  "";

	QString  ssid;
	for(int i = 0;  i < ireq->i_len;  ++i)
		ssid += (char) data[i];

	return  ssid;
}

/** Returns WEPOn when WEP encryption is used,
  * WEPOff if not and WIInvalid if an error occured.
  */
WEPMode  WirelessInterface::getWEPMode()  const {
	uint8_t  data[32];
	struct ieee80211req  *ireq = getIReq(IEEE80211_IOC_WEP,  (uint8_t **) &data);
	if(ireq == NULL)
		return  WIInvalid;

	WEPMode  mode = WIInvalid;
	switch(ireq->i_val) {
		case IEEE80211_WEP_NOSUP:
			mode = WIInvalid;  break;

		case IEEE80211_WEP_OFF:
			mode = WEPOff;  break;

		case IEEE80211_WEP_ON:
			mode = WEPOn;  break;

		case IEEE80211_WEP_MIXED:
			mode = WEPMixed;  break;
	}

	delete  ireq;
	return  mode;
}

/** Scans for wireless access points.
  *
  * @return QPtrList with the found access points.
  */
QPtrList<AccessPoint>  WirelessInterface::getAPs()  {
	/* Status must be UP to be able to scan for ap's. */

	if(up() < 0  && !isUp())
		/* up() failed and interface seems to be down,
		* so a scan would return nothing anyway.
		*/
		return  QPtrList<AccessPoint>();

	QPtrList<AccessPoint>  list;

	struct  wi_req  wreq;
	bzero(&wreq, sizeof(wreq));
	wreq.wi_len = WI_MAX_DATALEN;
	wreq.wi_type = WI_RID_READ_APS;

	if(wiGetVal(&wreq) < 0)
		return  list;

	struct  wi_apinfo  *apinfos;
	apinfos = (struct  wi_apinfo *) (((char *) &wreq.wi_val) + sizeof(int));

	int  cnt = * (int *) wreq.wi_val;
	for(int i = cnt - 1;  i >= 0;  --i) {
		bool  exists = false;
		for(int  n = cnt - 1;  n > i  &&  !exists  ;  --n) {
			exists |= (bcmp(apinfos[n].bssid,
							apinfos[i].bssid,
							sizeof(apinfos[0].bssid)) == 0);
		}

		if(!exists)
			list.append(new  AccessPoint(apinfos[i]));
	}

	return  list;
}

int  WirelessInterface::wiGetVal(struct  wi_req  *wreq)  const {
	if(wreq == NULL)
		return  -1;

	struct ifreq ifr;
	int  s, ret;

	bzero(&ifr, sizeof(ifr));

	strlcpy(ifr.ifr_name, getName(), sizeof(ifr.ifr_name));
	ifr.ifr_data = (caddr_t) wreq;

	s = socket(AF_INET, SOCK_DGRAM, 0);
	if(s == -1)
		return  -1;

	ret = ioctl(s, SIOCGWAVELAN, &ifr);
	close(s);

	return  ret;
}

/** Returns a list with all wireless network interfaces. */
QPtrList<WirelessInterface>  WirelessInterface::getWirelessIfs() {
	QPtrList<WirelessInterface>  lst;

	struct if_nameindex  *ifidx = if_nameindex();

	struct if_nameindex  *ifptr = ifidx;
	while(ifptr != NULL  &&  ifptr->if_name != NULL) {
		WirelessInterface  *netIf = new WirelessInterface(ifptr->if_name);

		if(netIf->isWireless() && !netIf->isLoopback())
			lst.append(netIf);
		else
			delete  netIf;

		++ifptr;
	}

	if_freenameindex(ifidx);
	return  lst;
}


int  WirelessInterface::set80211(int  type,  int  val,  int  length,  uint8_t *data)  const {
	int  s = newSocket();
	if(s < 0)
		return  -1;

	struct	ieee80211req  ireq;

	bzero(&ireq,  sizeof ireq);
	strlcpy(ireq.i_name,  getName().latin1(),  sizeof ireq.i_name);
	ireq.i_type = type;
	ireq.i_val = val;
	ireq.i_len = length;
	ireq.i_data = data;

	int  retval = ioctl(s, SIOCS80211, &ireq);
	close(s);
	return  retval;
}



/** @param data pointer to a pointer to a uint8_t array with minimum size 32 */
struct ieee80211req  *WirelessInterface::getIReq(int  type,  uint8_t  **data)  const {
	int  s = newSocket();
	if(s < 0)
		return  NULL;

	struct ieee80211req  *ireq = new ieee80211req;
	bzero(ireq,  sizeof *ireq);
	strlcpy(ireq->i_name,  getName(),  sizeof ireq->i_name);
	ireq->i_data = data;
	ireq->i_type = type;
	ireq->i_val = -1;

	if(ioctl(s, SIOCG80211, ireq) < 0) {
		close(s);
		delete  ireq;
		return  NULL;
	}

	close(s);
	return  ireq;
}

